#!/usr/bin/python
#
#   PROGRAM:     Firebird PowerConsole
#   MODULE:      attrdoc.py
#   DESCRIPTION: Documentation strings for metadata attributes
#
#  The contents of this file are subject to the Initial
#  Developer's Public License Version 1.0 (the "License");
#  you may not use this file except in compliance with the
#  License. You may obtain a copy of the License at
#  http://www.ibphoenix.com/main.nfs?a=ibphoenix&page=ibp_idpl.
#
#  Software distributed under the License is distributed AS IS,
#  WITHOUT WARRANTY OF ANY KIND, either express or implied.
#  See the License for the specific language governing rights
#  and limitations under the License.
#
#  The Original Code was created by Pavel Cisar
#  for the Firebird Open Source RDBMS project.
#  http://www.firebirdsql.org
#
#  Copyright (c) 2006 Pavel Cisar <pcisar@users.sourceforge.net>
#  and all contributors signed below.
#
#  All Rights Reserved.
#  Contributor(s): ______________________________________.

# table dictionary of field doc dictionaries

attrdoc = {
"RDB$CHARACTER_SETS":
   {"RDB$CHARACTER_SET_NAME":
        "Name of a character set known to Firebird.",
    "RDB$FORM_OF_USE":
        "Not used.",
    "RDB$NUMBER_OF_CHARACTERS":
        "Number of characters in the set (not used for the available character sets).",
    "RDB$DEFAULT_COLLATE_NAME":
        """Name of the binary collation sequence for the character set. It is 
always the same as the character set name.""",
    "RDB$CHARACTER_SET_ID":
        "Unique identifier for this character set, wherever it is used.",
    "RDB$SYSTEM_FLAG":
        """Will be 1 if the character set is defined by system at the database create;
0 for user-defined character set.""",
    "RDB$DESCRIPTION":
        "For storing documentation.",
    "RDB$FUNCTION_NAME":
        """Not used, but may become available for user-defined character sets that
are accessed via an external function.""",
    "RDB$BYTES_PER_CHARACTER":
        "Size of characters in the set, in bytes. For example FSS_UNICODE uses 3 bytes per character."
   },
"RDB$CHECK_CONSTRAINTS":
   {"RDB$CONSTRAINT_NAME":
        "Name of a constraint.",
    "RDB$TRIGGER_NAME":
        """For a CHECK constraint, this is the name of the trigger that enforces
 the constraint. For a NOT NULL constraint, this is the name of the column to
which the constraint applies - the table name can be found through the constraint name."""
    },
"RDB$COLLATIONS":
   {"RDB$COLLATION_NAME":
        "Name of the collation sequence.",
    "RDB$COLLATION_ID":
        "With the character set ID, unique collation identifier.",
    "RDB$CHARACTER_SET_ID":
        "With the collation ID, unique collation identifier.",
    "RDB$COLLATION_ATTRIBUTES":
        "Not used extrenally.",
    "RDB$SYSTEM_FLAG":
        "User-defined = 0; system-defined = 1 or higher.",
    "RDB$DESCRIPTION":
        "For storing documentation.",
    "RDB$FUNCTION_NAME":
        "Not currently used."
   }
}