#!/usr/bin/python
#
#   PROGRAM:     Firebird PowerConsole
#   MODULE:      stdcmd.py
#   DESCRIPTION: Standard built-in commands
#
#  The contents of this file are subject to the Initial
#  Developer's Public License Version 1.0 (the "License");
#  you may not use this file except in compliance with the
#  License. You may obtain a copy of the License at
#  http://www.ibphoenix.com/main.nfs?a=ibphoenix&page=ibp_idpl.
#
#  Software distributed under the License is distributed AS IS,
#  WITHOUT WARRANTY OF ANY KIND, either express or implied.
#  See the License for the specific language governing rights
#  and limitations under the License.
#
#  The Original Code was created by Pavel Cisar
#  for the Firebird Open Source RDBMS project.
#  http://www.firebirdsql.org
#
#  Copyright (c) 2006 Pavel Cisar <pcisar@users.sourceforge.net>
#  and all contributors signed below.
#
#  All Rights Reserved.
#  Contributor(s): ______________________________________.

#---Classes---#000000#FFFFFF----------------------------------------------------

from pyparsing import *
from fbpw.common import *
from fbpw.ibptoolbox import importModule, columnize, isiterable
from inspect import getdoc, currentframe
import sys
import types
import fbpw.fbcore as fb
import kinterbasdb as kdb
import Itpl

class pwCmdQuit(pwCommand):
    def __init__(self, shell):
        super(pwCmdQuit,self).__init__(shell)
        self.keyQuit = CaselessKeyword('quit')
        # QUIT
        self.cmdQuit = self.keyQuit.setResultsName('cmd')
        self.cmdQuit.setParseAction(self._cmdParseAction)

    def getGrammar(self):
        return self.cmdQuit
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyQuit
#    def execute(self):
        ## ToDo: Implement pwCmdQuit execute

class pwCmdExit(pwCommand):
    def __init__(self, shell):
        super(pwCmdExit,self).__init__(shell)
        self.keyExit = CaselessKeyword('exit')
        # EXIT
        self.cmdExit = self.keyExit.setResultsName('cmd')
        self.cmdExit.setParseAction(self._cmdParseAction)

    def getGrammar(self):
        return self.cmdExit
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyExit
#    def execute(self):
        ## ToDo: Implement pwCmdExit execute

class pwCmdIndexActionMixin(object):
    def __doIndexAction(self,index,action,errorMsg):
        if hasattr(index,'getSQLCommand'):
            cmd = index.getSQLCommand(action).replace('$','$$')
            self.shell.run(cmd, prompt=False, verbose=True)
        else:
            self.shell.writeErr(errorMsg % type(index).__name__)
    def _indexAction(self,line,action,errorMsg):
        if line == '':
            line = self.shell.getMainDatabaseName()
        try:
            obj = eval(line,self._getGlobals(),currentframe().f_back.f_back.f_back.f_locals)
        except Exception, e:
            self.shell.writeErr("%s : %s" % (str(e.__class__), str(e)))
            return
        if isinstance(obj,fb.Database):
            obj = obj.indices
        elif isinstance(obj,fb.Table):
            obj = obj.indices
        elif isinstance(obj,fb.Index):
            obj = (obj,)
        elif isinstance(obj,types.StringTypes):
            obj = (obj,)
        if isiterable(obj):
            self.shell.prompt()
            for o in obj:
                if isinstance(o,types.StringTypes):
                    name = o
                    o = self.shell.getMainDatabase().getIndex(name)
                    if o is None:
                        self.shell.writeErr("Index %s not found in database %s" % 
                            (name,self.shell.getMainDatabaseName()))
                        continue
                if isinstance(o,fb.Table):
                    for i in o.indices:
                        self.__doIndexAction(i,action,errorMsg)
                elif isinstance(o,fb.Index):
                    self.__doIndexAction(o,action,errorMsg)
                elif isinstance(o,fb.Database):
                    for i in o.indices:
                        self.__doIndexAction(i,action,errorMsg)
                else:
                    self.shell.writeErr(errorMsg % type(o).__name__)

class pwCmdRecompute(pwCommand,pwCmdIndexActionMixin):
    def __init__(self, shell):
        super(pwCmdRecompute,self).__init__(shell)
        self.keyRecompute = CaselessKeyword('recompute')
        # RECOMPUTE expr
        self.cmdRecompute = self.keyRecompute.setResultsName('cmd') + \
            self.expr.setResultsName('arg')
        self.cmdRecompute.setParseAction(self._cmdParseAction)

    def getGrammar(self):
        return self.cmdRecompute
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyRecompute
    def execute(self,arg):
        """Recomputes index statistics.

        usage: 
        recompute [<database>|<list_of_indices>|<list_of_names>]

        Without argument it would recompute all user indices in working database.
        If argument is a database object, it would recompute all user indices in it
        If argument is a table object, it would recompute indices for this table.
        If argument is a list of index or table objects, it would recompute these
        indices or indices associated to these tables.
        If argument is a list object, it must contain names of indices in current
        database to recompute.

        Examples:
        >>>recompute employee  # recompute all in employee database
        >>>recompute db.indices  # recompute all indices in database db
        >>>recompute db.table('JOB').indices # recompute indices of JOB table
        >>>recompute ['JOB_FK1','JOB_FK2']  # recomputes specified indices
        """
        self._indexAction(arg,'recompute',"Can't recompute statistics for '%s' object")

class pwCmdDeactivate(pwCommand,pwCmdIndexActionMixin):
    def __init__(self, shell):
        super(pwCmdDeactivate,self).__init__(shell)
        self.keyDeactivate = CaselessKeyword('deactivate')
        # DEACTIVATE expr
        self.cmdDeactivate = self.keyDeactivate.setResultsName('cmd') + \
            self.expr.setResultsName('arg')
        self.cmdDeactivate.setParseAction(self._cmdParseAction)

    def getGrammar(self):
        return self.cmdDeactivate
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyDeactivate
    def execute(self,arg):
        """Deactivates indices.

        usage: 
        deactivate [<database_obj>|<table_obj>|<index_obj>|<list>]

        Without argument it would deactivate all user indices in working database.
        If argument is a database object, it would deactivate all user indices in it
        If argument is a table object, it would deactivate indices for this table.
        If argument is list of index or table objects, it would deactivate these
        indices or indices associated to these tables.
        If argument is list object, it must contain names of indices in current
        database to deactivate.

        Examples:
        >>>deactivate employee  # deactivates all in employee database
        >>>deactivate db.indices  # deactivates all indices in database db
        >>>deactivate db.table('JOB').indices # deactivates indices of JOB table
        >>>deactivate ['JOB_FK1','JOB_FK2']  # deactivates specified indices
        """
        self._indexAction(arg,'deactivate',"Can't deactivate indices for '%s' object")

class pwCmdActivate(pwCommand,pwCmdIndexActionMixin):
    def __init__(self, shell):
        super(pwCmdActivate,self).__init__(shell)
        self.keyActivate = CaselessKeyword('activate')
        # ACTIVATE expr
        self.cmdActivate = self.keyActivate.setResultsName('cmd') + \
            self.expr.setResultsName('arg')
        self.cmdActivate.setParseAction(self._cmdParseAction)

    def getGrammar(self):
        return self.cmdActivate
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyActivate
    def execute(self,arg):
        """Deactivates indices.

        usage: 
        activate [<database_obj>|<table_obj>|<index_obj>|<list>]

        Without argument it would activate all user indices in working database.
        If argument is a database object, it would activate all user indices in it
        If argument is a table object, it would activate indices for this table.
        If argument is list of index or table objects, it would activate these
        indices or indices associated to these tables.
        If argument is list object, it must contain names of indices in current
        database to activate.

        Examples:
        >>>activate employee  # activates all in employee database
        >>>activate db.indices  # activates all indices in database db
        >>>activate db.table('JOB').indices # activates indices of JOB table
        >>>activate ['JOB_FK1','JOB_FK2']  # activates specified indices
        """
        self._indexAction(arg,'activate',"Can't activate indices for '%s' object")

class pwCmdRebuild(pwCommand,pwCmdIndexActionMixin):
    def __init__(self, shell):
        super(pwCmdRebuild,self).__init__(shell)
        self.keyRebuild = CaselessKeyword('rebuild')
        # REBUILD expr
        self.cmdRebuild = self.keyRebuild.setResultsName('cmd') + \
            self.expr.setResultsName('arg')
        self.cmdRebuild.setParseAction(self._cmdParseAction)

    def getGrammar(self):
        return self.cmdRebuild
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyRebuild
    def execute(self,arg):
        """Deactivates indices.

        usage: 
        rebuild [<database_obj>|<table_obj>|<index_obj>|<list>]

        Without argument it would rebuild all user indices in working database.
        If argument is a database object, it would rebuild all user indices in it
        If argument is a table object, it would rebuild indices for this table.
        If argument is list of index or table objects, it would rebuild these
        indices or indices associated to these tables.
        If argument is list object, it must contain names of indices in current
        database to rebuild.

        Examples:
        >>>rebuild employee  # rebuilds all in employee database
        >>>rebuild db.indices  # rebuilds all indices in database db
        >>>rebuild db.table('JOB').indices # rebuilds indices of JOB table
        >>>rebuild ['JOB_FK1','JOB_FK2']  # rebuilds specified indices
        """
        self._indexAction(arg,'deactivate',"Can't deactivate indices for '%s' object")
        self._indexAction(arg,'activate',"Can't activate indices for '%s' object")

class pwCmdHelp(pwCommand):
    ruler = '='
    doc_leader = """To get help about specific topic, PowerConsole command
or Python object/function/expression, type help <topic>.
"""
    doc_header = "PowerConsole commands:"
    misc_header = "Miscellaneous help topics:"
    nohelp = "*** No help on %s"
    
    tooltip = ('help', None, doc_leader+
    'Without arguments it will show the list of commands and known topics.')

    def __init__(self, shell):
        super(pwCmdHelp,self).__init__(shell)
        import __builtin__
        self._help = __builtin__.help
        __builtin__.help = self.execute
        self.renderer = shell.getRenderer(RENDERER_LINE)
        # Find and instantiate help topics
        self.help_providers = []
        mod = importModule('fbpw.stdhelp')
        for item in dir(mod):
            if (type(mod.__dict__[item]) == types.TypeType) and \
                issubclass(mod.__dict__[item],pwHelpProvider) and \
                (mod.__dict__[item] != pwHelpProvider):
                help = mod.__dict__[item](self._help)
                #print help.__class__.__name__
                self.help_providers.append(help)
        # Grammar
        self.keyHelp = CaselessKeyword('help')
        # HELP [arg]
        self.cmdHelp = self.keyHelp.setResultsName('cmd') + \
            Optional(self.arg.setResultsName('arg'))
        self.cmdHelp.setParseAction(self._cmdParseAction)

#---Private---#000000#FFFFFF----------------------------------------------------

    def __findProvider(self, topic):
        for provider in self.help_providers:
            if provider.canHandle(topic): 
                return provider
        return None
    def __showHelp(self,help):
        if callable(help):
            help()
        else:
            self.renderer.writeLine(help)
    def __printTopics(self, header, cmds, cmdlen, maxcol):
        if cmds:
            self.renderer.writeLine("%s" % str(header))
            if self.ruler:
                self.renderer.writeLine("%s" % str(self.ruler * len(header)))
            for line in columnize(cmds, maxcol-1):
                self.renderer.writeLine(line)
            self.renderer.writeLine()

#---Public---#000000#FFFFFF-----------------------------------------------------

    def getGrammar(self):
        return self.cmdHelp
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyHelp
    def execute(self,arg=''):
        """Todo: Documentation for help command itself."""
        self.renderer.initialize()
        if arg:
            if self.shell.commander.commands.has_key(arg):
                # Help for PowerConsole command
                cmd = self.shell.commander.commands[arg]
                try:
                    self.__showHelp(getattr(cmd, 'getHelp'))
                except AttributeError:
                    # There is not separate help provider function
                    # we'll look for docstring in 'execute' method
                    doc = getdoc(getattr(cmd, 'execute'))
                    if doc:
                        self.__showHelp(str(doc))
                    else:
                        self.__showHelp("%s" % str(self.nohelp % arg))
            else:
                # It's not a PowerConsole command, we'll look for help provider
                provider = self.__findProvider(arg)
                if provider:
                    self.__showHelp(provider.getHelp(arg))
                else:
                    try:
                        print self._getGlobals().keys()
                        print currentframe().f_back.f_back.f_locals.keys()
                        self._help(eval(arg,self._getGlobals(),currentframe().f_back.f_back.f_locals))
                    except NameError:
                        self.__showHelp("%s" % str(self.nohelp % arg))
        else:
            cmds_doc = []
            for cmd in self.shell.commander.commands.values():
                if hasattr(self, 'getHelp') or getdoc(getattr(cmd, 'execute')):
                    cmds_doc.append(cmd.getName())
            helptopics = []
            for provider in self.help_providers:
                helptopics.extend(provider.getTopics())
            cmds_doc.sort()
            helptopics.sort()
            self.renderer.writeLine("%s" % str(self.doc_leader))
            self.__printTopics(self.doc_header, cmds_doc, 15,80)
            self.__printTopics(self.misc_header, helptopics, 15,80)
        self.renderer.finalize()

class pwCmdConnect(pwCommand):
    def __init__(self, shell):
        super(pwCmdConnect,self).__init__(shell)
        self.keyConnect     = CaselessKeyword('connect')
        # we don't need to keep next tokens as instance attributes
        keyUser     = CaselessKeyword('user')
        keyPassword = CaselessKeyword('password')
        keyCache    = CaselessKeyword('cache')
        keyRole     = CaselessKeyword('role')
        keyAs       = CaselessKeyword('as')
        keyHost     = CaselessKeyword('host')
        optUser     = keyUser + QuotedString("'").setResultsName('user').setFailAction(self._fail)
        optPassword = keyPassword + QuotedString("'").setResultsName('password').setFailAction(self._fail)
        optCache    = keyCache + self.integer.setResultsName('cache').setFailAction(self._fail)
        optRole     = keyRole + QuotedString("'").setResultsName('role').setFailAction(self._fail)
        optAs       = keyAs + self.ident.setResultsName('as').setFailAction(self._fail)
        optHost     = keyHost + self.ident.setResultsName('host').setFailAction(self._fail)

        # CONNECT 'dsn' [USER 'user'][PASSWORD 'password']
        #  [CACHE int ][ROLE 'role'][AS var]
        self.cmdConnect  = self.keyConnect.setResultsName('cmd') + \
            self.filename.setResultsName('db') \
            + Optional(optHost) + Optional(optUser) + Optional(optPassword) \
            + Optional(optCache) + Optional(optRole) + Optional(optAs)
        self.cmdConnect.setParseAction(self._cmdParseAction)

    def getGrammar(self):
        return self.cmdConnect
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyConnect
    def execute(self,db,**kwargs):
        """Attach to the database.

        usage: 
        CONNECT 'dsn or db' [HOST host][USER 'user'][PASSWORD 'password']
          [CACHE int ][ROLE 'role'][AS var]

        If connection is sucessfuly established, this database connection is set
        as main working database. Database object is stored and accessible under
        default alias "db" or as alias specified by optional AS argument.
"""
        name = kwargs.get('as','db')
        user = kwargs.get('user','unknown')
        if kwargs.has_key('host'):
            kwargs['database'] = db
        else:
            kwargs['dsn'] = db
        try:
            d = fb.Database(**kwargs)
            self.shell.setDatabase(d, name)
            self.shell.writeOut('Database %s at %s, user %s' % (name, d.dsn, user))
        except kdb.OperationalError, e:
            self.shell.writeErr("Error: %s" % str(e[0]))
            self.shell.writeErr(str(e[1]))
#        except Exception, e:
#            self.shell.writeErr("%s : %s" % (str(e.__class__), str(e)))

class pwCmdShow(pwCommand):
    tooltip = ('show',None,"""CHECK dbident | 
DATABASE | DB | 
DOMAIN[S] [dbident] | 
EXCEPTION[S] [dbident] | 
FUNCTION[S] [dbident] | 
GENERATOR[S] [dbident] | 
GRANT [dbident] | 
INDEX | INDICES [dbident] | 
PROCEDURE[S] [dbident] | 
ROLE | 
SQL DIALECT | 
SYSTEM [TABLE[S]] | 
TABLE[S] [dbident] | 
TRIGGER[S] [dbident] | 
VERSION | 
VIEW[S] [dbident] | 
expression
""")
    def __init__(self, shell):
        super(pwCmdShow,self).__init__(shell)
        self.keyShow = CaselessKeyword('show')
        # we don't need to keep next tokens as instance attributes
        keyCheck       = CaselessKeyword('check')
        keyDatabase    = CaselessKeyword('database') #| CaselessKeyword('db')
        keyDomain      = CaselessKeyword('domain') | CaselessKeyword('domains')
        keyException   = CaselessKeyword('exception') | CaselessKeyword('exceptions')
        keyFunction    = CaselessKeyword('function') | CaselessKeyword('functions')
        keyGenerator   = CaselessKeyword('generator') | CaselessKeyword('generators')
        keyGrant       = CaselessKeyword('grant')
        keyIndices     = CaselessKeyword('index') | CaselessKeyword('indices')
        keyProcedure   = CaselessKeyword('procedure') | CaselessKeyword('procedures')
        keyRole        = CaselessKeyword('role')
        keySQL         = CaselessKeyword('sql')
        keyDialect     = CaselessKeyword('dialect')
        keySystem      = CaselessKeyword('system')
        keyTable       = CaselessKeyword('table') | CaselessKeyword('tables')
        keyTrigger     = CaselessKeyword('trigger') | CaselessKeyword('triggers')
        keyVersion     = CaselessKeyword('version')
        keyView        = CaselessKeyword('view') | CaselessKeyword('views')
        # SHOW [expr]
        self.cmdShow = self.keyShow.setResultsName('cmd') + \
            ( 
              (keyCheck.setResultsName('category').setParseAction(self._upper) + \
                self.dbident.setResultsName('objname').setFailAction(self._fail)) | 
              (keyDatabase.setResultsName('category').setParseAction(self._upper)) | 
              (keyDomain.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              (keyException.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              (keyFunction.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              (keyGenerator.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              (keyGrant.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              (keyIndices.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              (keyProcedure.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              (keyRole.setResultsName('category').setParseAction(self._upper)) | 
              (keySQL + keyDialect.setResultsName('category').setParseAction(self._upper)) | 
              (keySystem.setResultsName('category').setParseAction(self._upper) + \
                Optional(keyTable)) | 
              (keyTable.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              (keyTrigger.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              (keyVersion.setResultsName('category').setParseAction(self._upper)) | 
              (keyView.setResultsName('category').setParseAction(self._upper) + \
                Optional(self.dbident.setResultsName('objname'))) | 
              self.arg.setResultsName('arg')
            )
        self.cmdShow.setParseAction(self._cmdParseAction)

    def getGrammar(self):
        return self.cmdShow
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyShow
    def execute(self,arg=None,category=None,objname=None):
        """Display name(s) of database object(s).

        usage: 
        SHOW 
          CHECK dbident | 
          DATABASE | DB | 
          DOMAIN[S] [dbident] | 
          EXCEPTION[S] [dbident] | 
          FUNCTION[S] [dbident] | 
          GENERATOR[S] [dbident] | 
          GRANT [dbident] | 
          INDEX | INDICES [dbident] | 
          PROCEDURE[S] [dbident] | 
          ROLE | 
          SQL DIALECT | 
          SYSTEM [TABLE[S]] | 
          TABLE[S] [dbident] | 
          TRIGGER[S] [dbident] | 
          VERSION | 
          VIEW[S] [dbident] | 
          expression
        Examples:
        >>>show db.tables  # List names of all user tables in database
"""
        if category != None:
            db = self.shell.getMainDatabase()
            # Object(s) to show specified in ISQL way
            if category == 'CHECK':
                obj = None
            elif category in ('DATABASE','DB'):
                obj = None
            elif category.startswith('DOMAIN'):
                if objname == None:
                    obj = db.domains
                else:
                    obj = db.getDomain(objname)
            elif category.startswith('EXCEPTION'):
                if objname == None:
                    obj = db.exceptions
                else:
                    obj = db.getException(objname)
            elif category.startswith('FUNCTION'):
                obj = None
            elif category.startswith('GENERATOR'):
                if objname == None:
                    obj = db.generators
                else:
                    obj = db.getGenerator(objname)
            elif category == 'GRANT':
                obj = None
            elif category in ('INDEX','INDICES'):
                if objname == None:
                    obj = db.indices
                else:
                    obj = db.getIndex(objname)
            elif category.startswith('PROCEDURE'):
                if objname == None:
                    obj = db.procedures
                else:
                    obj = db.getProcedure(objname)
            elif category == 'ROLE':
                obj = None
            elif category == 'DIALECT':
                obj = None
            elif category == 'SYSTEM':
                obj = db.systables
            elif category.startswith('TABLE'):
                if objname == None:
                    obj = db.tables
                else:
                    obj = db.getTable(objname)
            elif category.startswith('TRIGGER'):
                if objname == None:
                    obj = db.triggers
                else:
                    obj = db.getTrigger(objname)
            elif category == 'VERSION':
                obj = None
            elif category.startswith('VIEW'):
                if objname == None:
                    obj = db.views
                else:
                    obj = db.getView(objname)
            if isinstance(obj,fb.BaseDBObject):
                renderer = self.shell.getRenderer(RENDERER_OBJECT)
                renderer.initialize()
                renderer.writeObject(obj)
                renderer.finalize()
            elif obj and isiterable(obj):
                l = [x.name for x in obj if hasattr(x,'name')]
                l.sort()
                if hasattr(obj[0],'_get_show_header'):
                    l.insert(0,obj[0]._get_show_header())
                renderer = self.shell.getRenderer(RENDERER_LIST)
                renderer.initialize()
                renderer.writeList(l)
                renderer.finalize()
            else:
                self.shell.writeOut("No database object to print.")
        elif arg:
            # Object(s) to show specified directly
            obj = eval(arg,self._getGlobals(),currentframe().f_back.f_back.f_locals)
            if isinstance(obj,fb.BaseDBObject):
                renderer = self.shell.getRenderer(RENDERER_OBJECT)
                renderer.initialize()
                renderer.writeObject(obj)
                renderer.finalize()
            elif obj and isiterable(obj):
                renderer = self.shell.getRenderer(RENDERER_OBJECTLIST)
                renderer.initialize()
                renderer.writeList(obj)
                renderer.finalize()
            else:
                self.shell.writeOut("No database object to print.")

class pwCmdSQL(pwCommand):
    SQL_PARAM_SEPARATOR   = '<<'
    SQL_STORE_SEPARATOR   = '>>'
    
    def __init__(self, shell):
        super(pwCmdSQL,self).__init__(shell)
##        self.renderer = self.shell.getRenderer(RENDERER_RESULT)
        
        self.keySQL = CaselessKeyword('sql')
        self.cmdSQL = self.keySQL.setResultsName('cmd') + \
            White() + self.arg.setResultsName('sql')
        self.cmdSQL.setParseAction(self._cmdParseAction)
        # SQL commands
        self.keyCommit   = CaselessKeyword('commit')
        self.keyRollback = CaselessKeyword('rollback')
        self.keyAlter    = CaselessKeyword('alter')
        self.keyCreate   = CaselessKeyword('create')
        self.keyDelete   = CaselessKeyword('delete')
        self.keyDrop     = CaselessKeyword('drop')
        self.keyExecute  = CaselessKeyword('execute')
        self.keyGrant    = CaselessKeyword('grant')
        self.keyInsert   = CaselessKeyword('insert')
        self.keyRecreate = CaselessKeyword('recreate')
        self.keyRevoke   = CaselessKeyword('revoke')
        self.keySavepoint= CaselessKeyword('savepoint')
        self.keySelect   = CaselessKeyword('select')
        self.keyUpdate   = CaselessKeyword('update')
        self.keySet      = CaselessKeyword('set')

        # Second level keywords
        self.keyGenerator   = CaselessKeyword('generator')
        self.keyStatistics  = CaselessKeyword('statistics')
        
        # Composite SQL commands
        self.cmdSetGen      = (self.keySet + self.keyGenerator)
        self.cmdSetStat     = (self.keySet + self.keyStatistics)

#---Public---#000000#FFFFFF-----------------------------------------------------

    def getGrammar(self):
        return self.cmdSQL
    def getSQL(self):
        return (self.keyCommit | self.keyRollback | self.keyAlter | 
                self.keyCreate | self.keyDelete | self.keyDrop | 
                self.keyExecute | self.keyGrant | self.keyInsert | 
                self.keyRecreate | self.keyRevoke | self.keySavepoint |
                self.keySelect | self.keyUpdate | self.cmdSetGen |
                self.cmdSetStat
            )
    def getCmdKeyword(self):
        return self.keySQL
    def execute(self,sql,**kwargs):
        self.renderer = self.shell.getRenderer(RENDERER_RESULT)
        line = Itpl.itplns(sql, self._getGlobals(), currentframe().f_back.f_back.f_locals)
#        self.shell.writeOut("SQL command:%s" % line)
        try:
            # check for store variable
            if line.find(self.SQL_STORE_SEPARATOR) >= 0:
                line,var = line.split(self.SQL_STORE_SEPARATOR)
                kwargs['var'] = var.strip()
            # Check for input parameters
            if line.find(self.SQL_PARAM_SEPARATOR) >= 0:
                sql,params = line.split(self.SQL_PARAM_SEPARATOR)
                try:
                    params = eval(params,self._getGlobals(),currentframe().f_back.f_back.f_locals)
                    kwargs['params'] = params
                except Exception, e:
                    self.shell.writeErr("%s:%s" % (e.__class__, e))
                    return ''
            c = self.shell.getMainDatabase().cursor()
            if kwargs.has_key('var'):
                currentframe().f_back.f_back.f_locals[kwargs['var']] = c
            if kwargs.has_key('params'):
                # We have input parameters
                params = kwargs['params']
                if isinstance(params,(kdb.Cursor,types.GeneratorType)) or \
                  (sql.upper().startswith('SELECT') and \
                  isinstance(params[0],(types.ListType,types.TupleType))):
                    for par in params:
                        c.execute(sql,par)
                        if not kwargs.has_key('var'):
                            self.renderer.initialize()
                            self.renderer.writeResult(c,sql)
                            self.renderer.finalize()
                elif isinstance(params,(types.ListType,types.TupleType)) and\
                    isinstance(params[0],(types.ListType,types.TupleType)):
                    c.executemany(sql,params)
                else:
                    c.execute(sql,params)
                    if not kwargs.has_key('var'):
                        self.renderer.initialize()
                        self.renderer.writeResult(c,sql)
                        self.renderer.finalize()
            else:
                c.execute(line)
                if not kwargs.has_key('var'):
                    self.renderer.initialize()
                    self.renderer.writeResult(c,line)
                    self.renderer.finalize()
        except kdb._ALL_EXCEPTION_CLASSES, e:
            self.shell.writeErr(str(e[1]))
        except Exception,e:
            self.shell.writeErr("%s : %s" % (str(e.__class__), str(e)))

class pwReady(pwCommand):
    def __init__(self, shell):
        super(pwReady,self).__init__(shell)
        self.keyReady = CaselessKeyword('ready')
        # READY [<dbvar>]
        self.cmdReady = self.keyReady.setResultsName('cmd') + \
            Optional(self.expr.setResultsName('arg'))
        self.cmdReady.setParseAction(self._cmdParseAction)

    def getGrammar(self):
        return self.cmdReady
    def getSQL(self):
        return None
    def getCmdKeyword(self):
        return self.keyReady
    def execute(self,arg=None):
        """Show list of attached databases or select the working database.

        usage: 
        READY [var]
        
        Without argument, it will list all attached databases.
        If argument is specified, it must be a name of previously connected
        database (implicit name 'db' or name specified by AS argument).
"""
        if arg:
            if self.shell.getDatabases().has_key(arg):
                self.shell.setMainDatabase(arg)
                print "Main database set to",arg
            else:
                print "Unknown database '%s'" % arg
        else:
            print 'Attached databases:'
            for k,v in self.shell.getDatabases().items():
                print "Database %s at %s, user %s" % (k,v.dsn,v.user)
            db = self.shell.getMainDatabaseName()
            print
            if not db:
                print "Main database was not specified."
            else:
                print 'Main database is',db



##class pwCmd(pwCommand):
##    def __init__(self, shell):
##        super(pwCmd,self).__init__(shell)
##
##    def getGrammar(self):
##        return self.cmd
##    def getSQL(self):
##        return None
##    def getCmdKeyword(self):
##        return self.key
##    def execute(self,**kwargs):
##        """ToDo: Documentation for command cmd."""
##        ## ToDo: Implement pwCmd execute
##        print "Method pwCmd.execute not implemented"
